@@ -1,13 +1,15 @@
 {-
    Brick.hs (adapted from ogl2brick.c which is (c) 3Dlabs Inc. Ltd.)
-   Copyright (c) Sven Panne 2006 <sven.panne@aedion.de>
+   Copyright (c) Sven Panne 2006 <svenpanne@gmail.com>
    This file is part of HOpenGL and distributed under a BSD-style license
    See the file libraries/GLUT/LICENSE
 -}
 
 import Prelude hiding ( sum )
 import Control.Applicative
-import Control.Monad
+import Control.Exception ( IOException, catch )
+import Control.Monad ( when, unless )
+import qualified Data.ByteString as B
 import Data.Foldable ( Foldable, sum )
 import Data.IORef
 import System.Exit
@@ -288,11 +290,11 @@ checkGLSLSupport = do
       unless ("GL_ARB_shading_language_100" `elem` extensions) $
          ioError (userError "No GLSL support found.")
 
-readAndCompileShader :: Shader s => FilePath -> IO s
-readAndCompileShader filePath = do
-   src <- readFile filePath
-   [shader] <- genObjectNames 1
-   shaderSource shader $= [src]
+readAndCompileShader :: ShaderType -> FilePath -> IO Shader
+readAndCompileShader st filePath = do
+   src <- B.readFile filePath
+   shader <- createShader st
+   shaderSourceBS shader $= src
    compileShader shader
    reportErrors
    ok <- get (compileStatus shader)
@@ -303,10 +305,10 @@ readAndCompileShader filePath = do
       ioError (userError "shader compilation failed")
    return shader
 
-installBrickShaders :: [VertexShader] -> [FragmentShader] -> IO ()
-installBrickShaders vs fs = do
-   [brickProg] <- genObjectNames 1
-   attachedShaders brickProg $= (vs, fs)
+installBrickShaders :: [Shader] -> IO ()
+installBrickShaders shaders = do
+   brickProg <- createProgram
+   attachedShaders brickProg $= shaders
    linkProgram brickProg
    reportErrors
    ok <- get (linkStatus brickProg)
@@ -331,10 +333,10 @@ installBrickShaders vs fs = do
 
 main :: IO ()
 main = do
-   getArgsAndInitialize
+   _ <- getArgsAndInitialize
    initialDisplayMode $= [ RGBMode, WithDepthBuffer, DoubleBuffered ]
    initialWindowSize $= Size 500 500
-   createWindow "3Dlabs Brick Shader"
+   _ <- createWindow "3Dlabs Brick Shader"
 
    -- Note: We don't use an idle callback, we redisplay more intelligently.
    state <- makeState
@@ -344,13 +346,13 @@ main = do
    motionCallback $= Just (motion state)
    addTimerCallback timerFrequencyMillis (timer state)
 
-   catch
+   Control.Exception.catch
      (do checkGLSLSupport
-         vs <- readAndCompileShader "Brick.vert"
-         fs <- readAndCompileShader "Brick.frag"
-         installBrickShaders [vs] [fs])
+         vs <- readAndCompileShader VertexShader "Brick.vert"
+         fs <- readAndCompileShader FragmentShader "Brick.frag"
+         installBrickShaders [vs, fs])
      (\exception -> do
-         print exception
+         print (exception :: IOException)
          putStrLn "Using fixed function pipeline."
          materialDiffuse Front $= Color4 1 0.3 0.2 1
          materialSpecular Front $= Color4 0.3 0.3 0.3 1
